@@ -28,7 +28,7 @@ use constant EPP_XMLNS	=> 'urn:ietf:params:xml:ns:epp-1.0';
 use vars qw($Error $Code $Message);
 
 BEGIN {
-	our $VERSION = '0.05_01';
+	our $VERSION = '0.06_01';
 }
 
 # file-scoped lexicals
@@ -219,14 +219,7 @@ sub new {
 	bless($self, $class);
 
 	# Connect to server
-	eval { $self->{greeting} = $self->connect (%params); };
-	unless ($self->{greeting}) {
-		$self->{connected} = 0;
-		warn 'No greeting returned: cannot continue';
-		warn ($@) if $@;
-		return undef;
-	}
-	$self->{connected}      = 1;
+	$self->_go_connect (%params) or return;
 
 	# Login
 	unless (defined $params{login} and $params{login} == 0) {
@@ -238,6 +231,25 @@ sub new {
 	return $Error ? undef : $self;
 }
 
+sub _go_connect {
+	my ($self, %params) = @_;
+	if (scalar keys %params) {
+		$self->{connect_params} = \%params;
+	} else {
+		%params = %{$self->{connect_params}};
+	}
+
+	# Connect to server
+	eval { $self->{greeting} = $self->connect (%params); };
+	unless ($self->{greeting}) {
+		$self->{connected} = 0;
+		warn 'No greeting returned: cannot continue';
+		warn ($@) if $@;
+		return undef;
+	}
+	$self->{connected}      = 1;
+}
+
 =pod
 
 =head1 Login
@@ -323,6 +335,7 @@ sub login {
 	}
 
 	$self->{authenticated} = 1;
+	$self->{login_params} = [$user, $pass, $options];
 	return $self;
 }
 
@@ -871,7 +884,7 @@ eg:
 	my $changes = {
 		'name'         => 'foo.co.uk',
 		'add'          => {
-			secDNS => [{ 
+			secDNS => [{
 				keyTag     => 25103,
 				alg        => 5,
 				digestType => 1,
@@ -949,7 +962,7 @@ sub modify_domain {
 	# DNSSEC
 	# Nominet does not support MaxSigLife which is the only possible use
 	# for 'chg', so do not cater for it here (yet).
-	# 
+	#
 	my $exttype = 'secDNS';
 	@spec  = $self->spec ($exttype);
 	my $obj2  = $frame->addObject (@spec);
@@ -1476,10 +1489,10 @@ sub hello {
 	}
 	my $frame = Net::EPP::Frame::Hello->new;
 
-	warn "Sending XML = \n" . $frame . "\n" if $Debug;
+	warn "Sending XML = \n" . $frame . "\n" if $Debug > 1;
 	my $greeting = $self->request($frame);
-	warn "Response XML = \n" . $greeting->toString() . "\n" if ($Debug
-	and defined $greeting);
+	warn "Response XML = \n" . $greeting->toString() . "\n"
+		if ($Debug > 1 && defined $greeting);
 
 	unless ($greeting) {
 		$Error = sprintf("Server returned a %d code", $Code);
@@ -1687,6 +1700,21 @@ sub _send_frame {
 		$Code   = 0;
 		$Error  = "No response from server";
 		warn $Error;
+		$self->{connected}     = 0;
+		$self->{authenticated} = 0;
+		if ($self->{reconnect}) {
+			# Attempt to reconnect
+			for (1 .. $self->{reconnect}) {
+				$self->_go_connect and
+				$self->login (@{$self->{login_params}});
+				if ($self->{authenticated}) {
+					$frame->clTRID->firstChild->setData ('');
+					return $self->_send_frame ($frame);
+				}
+				warn "Re-connection attempt $_ of $self->{reconnect} failed.\n";
+				sleep 2;
+			}
+		}
 		return undef;
 	}
 	warn "Response = " . $response->toString . "\n" if $Debug > 1;
@@ -1731,7 +1759,7 @@ are not yet supported.
 =item * Nominet's L<EPP
 Documentation|https://registrars.nominet.uk/uk-namespace/registration-and-domain-management/registration-systems/epp/>
 
-=item * The EPP RFCs: L<RFC 5730|https://tools.ietf.org/html/rfc5730>, 
+=item * The EPP RFCs: L<RFC 5730|https://tools.ietf.org/html/rfc5730>,
 L<RFC 5731|https://tools.ietf.org/html/rfc5731>,
 L<RFC 5732|https://tools.ietf.org/html/rfc5732> and
 L<RFC 5733|https://tools.ietf.org/html/rfc5733>.
@@ -1744,7 +1772,7 @@ Pete Houston <cpan@openstrike.co.uk>
 
 =head1 Licence
 
-This software is copyright © 2013-2021 by Pete Houston. It is released
+This software is copyright © 2013-2022 by Pete Houston. It is released
 under the Artistic Licence (version 2) and the
 GNU General Public Licence (version 2).
 
